;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  	Train Jumper Clone                                         /
;/                              Written by Donald Straney 7/23/04                                  /
;/                        Designed for the Z80/TI-86 graphing calculator                           /
;///////////////////////////////////////////////////////////////////////////////////////////////////

#include "86.inc"

;Game control
Game_Speed			equ	30	;40			;Game speed (delay in between redraws)
Sign_Delay			equ	25				;Redraws in between sign appearances
Car_Move			equ	4				;Number of pixels the cars move each refresh when running
Jump_Move			equ	4	;4,2,2,3,4	;Number of pixels the person moves each jump cycle
Fall_Move			equ	6				;Number of pixels the person moves each fall cycle
Sign_Move			equ	8				;Number of pixels signs move each refresh
Person_Land_Pos		equ	18				;Position where person lands
Person_Jump_Pos		equ	6	;2,8,10,6,6	;Position of top of jump
Person_Fall_Pos		equ	48				;Position of bottom of fall
Car_Score			equ	10				;Amount added to score when user jumps to next car
High_Sign_Score		equ	2				;Amount added to score when user passes signs
Low_Sign_Score		equ	5

;Gfx sizes
Car_Width			equ	12				;Dimensions of cars (width in bytes)
Car_Height			equ	25
Wheel_Height		equ	3
Post_Height			equ	27				;Height of post on signs
Person_Width		equ	8				;Width of solid part of person
Person_Jump_Height	equ	15				;Height of solid part of jumping person
Sign_Width			equ	8				;Width of solid part of signs
Sign_Height			equ	5				;Height of solid part of low sign

;Sprite positions
Sign_Init_Pos		equ	18				;Starting position of signs
Car_Init_Pos		equ	33*256+33		;Starting positions for cars
Person_Init_Pos		equ	100*256+18		;and person

;Cursor positions
Title_Pos			equ	$0402			;Position of title text
Title_2_Pos			equ	$3301			;Positions of credit text
Title_3_Pos			equ	$3901
High_Score_Pos		equ	$1827			;Position where high score is displayed
New_High_Pos		equ	$1823			;Position where "new high score!" text is displayed
Score_Pos			equ	$0001			;Position where score is displayed
Lose_Pos			equ	$0602			;Position of "you lose" text

;Vars
Car_Pos				equ	$8c00	;2		;Position of cars
Person_Pos			equ	$8c02	;2		;Position of person
Person_Sprite		equ	$8c04	;2		;Address of sprite to use when drawing person
Jump_State			equ	$8c06	;1		;0=standing, 1=going up, 2=coming down
Fall_State			equ	$8c07	;1		;0=not falling, 1=falling
Sign_Counter		equ	$8c08	;1		;Delay counter for signs
Sign_Type			equ	$8c09	;1		;0=no sign, 1=low sign, 2=high sign
Sign_Pos			equ	$8c0a	;2		;Position of current sign
Score				equ	$8c0c	;2		;Current score
Jumped_To_Car		equ	$8c0e	;1		;0=jumped to same car, 1=jumped to next car
Do_Redraw			equ	$8c0f	;1		;1 if needs to redraw screen

;Other addresses
ISR_Addr			equ	$8f8f			;Start of ISR
Vector_Table_Addr	equ	$8e00			;Start of ISR vector table
Back_Buffer_1		equ	$9000			;Grayscale buffers for background image
Back_Buffer_2		equ	$9d00

.org _asm_exec_ram

    nop
    jp Prog_Start

    .dw 0,Title_Str

Title_Str: .db "Train Jumper",0
Title_2_Str: .db "Written by Donald Straney",0
Title_3_Str: .db "straney@umbi.umd.edu",0
Lose_Str: .db "You fell",0
High_Score_Str: .db "High score: ",0
New_High_Str: .db "New high score!",0

data_start:								;Area to save
High_Score: .dw 0						;Current high score
data_end:

Prog_Start:
	call Wait_For_No_Key				;First wait until no keypresses, so they won't overflow into the title screen key handler
	call _flushallmenus					;Normal init...
	ld hl,Back_Layer2					;Put background image in buffers
	ld de,Back_Buffer_1
	ld bc,1024
	ldir
	ld hl,Back_Layer1
	ld de,Back_Buffer_2
	ld bc,1024
	ldir
	call Open_Gray						;and open grayscale handler
	ld hl,Car_Init_Pos					;Init gfx vars
	ld (Car_Pos),hl
	ld hl,Person_Init_Pos
	ld (Person_Pos),hl
	ld hl,Person_Still_Sprite
	ld (Person_Sprite),hl
	ld hl,$0000
	ld (Score),hl
Title_Loop:								;Now show title screen
	ld a,(Do_Redraw)
	or a
	jr z,Continue_Title_Loop
	call Redraw
	ld hl,Title_Pos						;and title text
	ld (_curRow),hl
	ld hl,Title_Str
	call _puts
	set textInverse,(iy+textflags)
	ld hl,Title_2_Pos
	ld (_penCol),hl
	ld hl,Title_2_Str
	call _vputs
	ld hl,Title_3_Pos
	ld (_penCol),hl
	ld hl,Title_3_Str
	call _vputs
	res textInverse,(iy+textflags)
	ld hl,High_Score_Pos				;and current high score
	ld (_penCol),hl
	ld hl,High_Score_Str
	call _vputs
	ld hl,(High_Score)
	call DisplayHL
Continue_Title_Loop:
	ld a,%00111111						;Check for keypress
	call Get_Key
	cp %10111111
	jp z,Close_Gray						;If exit was pressed, close grayscale handler and exit
	cp %11011111
	jr z,Start_Game						;If 2nd was pressed, start game!
	jr Title_Loop						;If none of these, loop again

Start_Game:
	call Wait_For_No_Key				;First wait until 2nd has been released
	xor a								;Init game vars
	ld (Jump_State),a
	ld (Fall_State),a
	ld (Sign_Type),a
	ld (Score),a
	ld (Score+1),a
	ld (Jumped_To_Car),a
	ld (Patch6),a						;enable score
	ld a,Sign_Delay
	ld (Sign_Counter),a
	call Redraw_Back					;and draw screen
	call Redraw_Front

;///////////////////////////////////////////////Main loop//////////////////////////////////////////////

Loop:
	ld a,(Do_Redraw)						;Are we supposed to redraw the gfx?
	or a
	call nz,Redraw						;If so, do it
	ld a,(Fall_State)					;Is the person falling?
	or a
	jp nz,Handle_Fall					;If so, show fall
	ld a,(Jump_State)					;Is the person jumping?
	or a
	call nz,Handle_Jump					;If so, show it
	ld a,%00111111						;Now check for keypresses
	call Get_Key
	cp %10111111
	jr z,Exit_Prog						;If exit was pressed, close grayscale handler and exit
	cp %11011111						;If 2nd was pressed, make person jump!
	call z,Try_Jump
	cp %01111111						;If More was pressed, pause
	call z,Pause
	ld a,%01111110
	call Get_Key
	cp %11111101
	jr z,Move_Left						;If left was pressed, make person run
	cp %11111110
	jr z,Duck							;If down was pressed, make person duck
	ld hl,Person_Still_Sprite			;Otherwise, show that person is standing still
	ld (Person_Sprite),hl
	jr Loop

Exit_Prog:
	call Close_Gray						;Close grayscale handler
	call Wait_For_No_Key				;Wait until no keys are pressed
	jp _clrScrn							;Clear screen and text shadow, then return

;////////////////////////////////////////////////Actions//////////////////////////////////////////////

Pause:
	ld a,%00111111						;Wait until 2nd is pressed
	call Get_Key
	cp %11011111
	jr nz,Pause
	jp Wait_For_No_Key					;then wait until it isn't pressed anymore before returning

Move_Left:
	ld a,(Jump_State)					;If jumping, don't show running person
	or a
	jr nz,Move_Left_Skip_2
;	ld hl,Person_Run_Sprite				;If running, show the person running
;	ld (Person_Sprite),hl
	ld hl,(Person_Sprite)				;Switch sprites to animate running
	ld de,Person_Run_Sprite
	call _cphlde
	jr z,Move_Left_Person_2
Move_Left_Person_1:
	ld hl,Person_Run_Sprite
	jr Move_Left_Set_Person
Move_Left_Person_2:
	ld hl,Person_Run_2_Sprite
Move_Left_Set_Person:
	ld (Person_Sprite),hl
Move_Left_Skip_2:
	ld a,(Car_Pos+1)					;and move the cars right
	add a,Car_Move
	cp 127
	jr c,Move_Left_Skip
	sub 127
Move_Left_Skip:
	ld (Car_Pos+1),a
	call Test_Fall						;See if the person has fallen
Move_End_Loop:
	ld a,(Do_Redraw)					;Now wait until the next redraw period
	or a
	jr z,Move_End_Loop
	call Redraw
	jp Loop

Duck:
	ld hl,Person_Duck_Sprite			;Show person ducking
	ld (Person_Sprite),hl
	jp Loop

;///////////////////////////////////////////////Jump////////////////////////////////////////////////

Try_Jump:
	ld a,(Jump_State)					;Unless the person isn't jumping, we're not interested
	or a
	ret nz
	inc a								;If the person isn't jumping, start jump
	ld (Jump_State),a
	ret

Handle_Jump:
	ld hl,Person_Jump_Sprite			;Show jumping person
	ld (Person_Sprite),hl
	push af
	ld a,(Person_Pos+1)					;See if person has jumped off car
	add a,Person_Width
	ld b,a
	ld a,(Car_Pos+1)
	cp b
	jr c,Continue_Jump					;If not, continue jump
	ld a,1
	ld (Jumped_To_Car),a				;Otherwise, show that person has jumped to next car
Continue_Jump:
	pop af
	dec a								;Is the person going up?
	jr z,Handle_Jump_Up
Handle_Jump_Down:						;If not, they must be coming down
	ld a,(Person_Pos)					;If coming down, move downwards 1 pixel
	add a,Jump_Move
	ld (Person_Pos),a
	cp Person_Land_Pos					;Has the person landed?
	jr nz,Do_Fast_Jump_Down				;If not, return
Hit_Bot_Jump:
	xor a
	ld (Jump_State),a
	call Test_Fall						;See if person has landed between cars, and set fall var if so
	ld a,(Jumped_To_Car)				;Has person landed on new car?
	or a
	ret z								;If not, return
	ld hl,(Score)						;Otherwise, increment score
	ld bc,Car_Score
	add hl,bc
	ld (Score),hl
	xor a
	ld (Jumped_To_Car),a				;and reset var
	ret
Handle_Jump_Up:
	ld a,(Person_Pos)					;If going up, move upwards 1 pixel
	sub Jump_Move
	ld (Person_Pos),a
	cp Person_Jump_Pos					;Is the person at the top of the jump?
	jr nz,Do_Fast_Jump_Up				;If not, return
Hit_Top_Jump:
	ld a,2								;If so, start coming down
	ld (Jump_State),a
	ret
Do_Fast_Jump_Up:						;Do faster-than-normal jump, only if not moving left
	ld a,%01111110						;Test if moving left
	call Get_Key
	cp %11111101
	ret z								;If so, return

;	ld a,(Person_Pos)					;Otherwise, move upwards again
;	sub Jump_Move
;	ld (Person_Pos),a
;	cp Person_Jump_Pos
;	jr z,Hit_Top_Jump

Fast_Jump_Up_End_Loop:					;Now wait for next redraw cycle
	ld a,(Do_Redraw)
	or a
	jr z,Fast_Jump_Up_End_Loop
	jr Redraw
Do_Fast_Jump_Down:						;Do faster-than-normal jump, only if not moving left
	ld a,%01111110						;Test if moving left
	call Get_Key
	cp %11111101
	ret z								;If so, return
	ld a,(Person_Pos)					;Otherwise, move down again
	add a,Jump_Move
	ld (Person_Pos),a
	cp Person_Land_Pos
	jr z,Hit_Bot_Jump
Fast_Jump_Down_End_Loop:				;Now wait for next redraw cycle
	ld a,(Do_Redraw)
	or a
	jr z,Fast_Jump_Down_End_Loop
;	jr Redraw

;///////////////////////////////////////////////Draw////////////////////////////////////////////////

Redraw:
	xor a								;Indicate that redraw has started
	ld (Do_Redraw),a
	ld hl,Sign_Counter					;Decrement sign counter
	dec (hl)
	call z,Start_Sign					;If it has timed out, reset it and draw sign
	ld b,64								;Now shift background images in buffers
	ld hl,Back_Buffer_1
	call Shift
	ld b,64
	ld hl,Back_Buffer_2
	call Shift
	call Redraw_Back					;copy them to the screen
	ld a,(Sign_Type)					;If there's a sign, then draw it
	or a
	call nz,Draw_Sign
;	jp Redraw_Front						;then draw the car and person on top of the background

Redraw_Front:							;Draw train
	xor a
	call Redraw_Front_Layer
	ld a,$ca-$fc
Redraw_Front_Layer:
	ld (Patch3),a
	ld bc,(Car_Pos)
	call FindPixel
	ld (Patch2),a
	ld (Patch4),a
	ld (Patch5),a
	ld de,16
	ld b,Car_Height
	push hl
Draw_Left_Edge:
	ld c,a
	or (hl)
	ld (hl),a
	ld a,c
	add hl,de
	djnz Draw_Left_Edge
	inc hl
	call Draw_Wheel
	pop hl
	inc hl
	ld a,Car_Width
Draw_Middle:
	push hl
	ld b,Car_Height
Draw_Middle_Col:
	ld (hl),$ff
	add hl,de
	djnz Draw_Middle_Col
	pop hl
	inc hl
	dec a
	jr nz,Draw_Middle
	ld b,Car_Height
Patch2	equ	$+1
	ld a,0
	cpl
Draw_Right_Edge:
	ld c,a
	or (hl)
	ld (hl),a
	ld a,c
	add hl,de
	djnz Draw_Right_Edge
	dec hl
	dec hl
	call Draw_Wheel
Draw_Person:							;and person
	ld hl,(Person_Sprite)
	ld bc,(Person_Pos)
	push hl
	push bc
	call GraySprite
	pop bc
	pop hl
	ld de,24
	add hl,de
	ld a,c
	add a,8
	ld c,a
	call GraySprite
Patch6	equ	$
	ret									;(will be changed to "nop" when game starts to enable score display)
	ld hl,Score_Pos						;and score
	ld (_penCol),hl
	ld hl,(Score)
	jp DisplayHL

Redraw_Back:							;Copy images from background buffers to screen
	ld hl,Back_Buffer_1
	ld de,$fc00
	ld bc,1024
	ldir
	ld hl,Back_Buffer_2
	ld de,$ca00
	ld bc,1024
	ldir
	ret

Draw_Wheel:
	push hl
Patch4	equ	$+1
	ld a,0
	ld b,Wheel_Height
Draw_Wheel_1:
	ld c,a
	or (hl)
	ld (hl),a
	ld a,c
	add hl,de
	djnz Draw_Wheel_1
	pop hl
	inc hl
Patch5	equ	$+1
	ld a,0
	cpl
	ld b,Wheel_Height
Draw_Wheel_2:
	ld c,a
	or (hl)
	ld (hl),a
	ld a,c
	add hl,de
	djnz Draw_Wheel_2
	ret

;///////////////////////////////////////////////Signs///////////////////////////////////////////////

Start_Sign:
	ld a,Sign_Delay						;Re-init sign counter
	ld (Sign_Counter),a
	ld hl,Sign_Init_Pos					;and sign position
	ld (Sign_Pos),hl
	ld a,r								;Now generate random number and use to select sign type
	cp 63
	jr c,Start_Sign_1
Start_Sign_2:
	ld a,2
	ld (Sign_Type),a
	ret
Start_Sign_1:
	ld a,1
	ld (Sign_Type),a
	ret

Draw_Sign:
	dec a								;Is it a low sign?
	jr z,Draw_Low_Sign
Draw_High_Sign:							;If not, it must be a high sign
	ld hl,Sign_High_Sprite				;Depending on which it is, select the correct sprite
	jr Draw_Sign_Continue
Draw_Low_Sign:
	ld hl,Sign_Low_Sprite
Draw_Sign_Continue:
	ld bc,(Sign_Pos)					;and draw it
	push hl
	push bc
	call GraySprite
	pop bc
	pop hl
	ld de,24
	add hl,de
	ld a,c
	add a,8
	ld c,a
	call GraySprite
	xor a
	call Draw_Post						;Draw post
	ld a,$ca-$fc
	call Draw_Post
	ld a,(Sign_Pos+1)					;Test if person has hit sign
	ld b,a
	ld a,(Person_Pos+1)
	sub b
	cp Sign_Width						;If they have, see if they were ducking/jumping to avoid it
	call c,Test_Sign
	ld a,(Sign_Pos+1)					;Now move sign to right
	add a,Sign_Move
	ld (Sign_Pos+1),a
	cp 128								;Is it off screen?
	ret c								;If not, end here
	xor a
	ld (Sign_Type),a					;If it is, kill sign
	ret

Draw_Post:
	ld (Patch3),a
	ld bc,(Sign_Pos)
	ld a,c
	add a,16
	ld c,a
	call FindPixel
	ld a,b
	ld b,Post_Height
	ld de,16
Draw_Post_Loop:
	ld c,a
	or (hl)
	ld (hl),a
	ld a,c
	add hl,de
	djnz Draw_Post_Loop
	ret

Test_Sign:
	ld a,(Sign_Type)					;See if it was a high or low sign
	dec a
	jr z,Test_Low_Sign
Test_High_Sign:
	ld a,(Jump_State)					;If it was a high sign, was person jumping?
	or a
	jr nz,Fall							;If so, they fall
	ld a,%01111110						;Were they ducking?
	call Get_Key
	cp %11111110
	jr z,Passed_High					;If so, the sign passed overhead
	jr Fall								;Otherwise, they fall
Test_Low_Sign:
	ld a,(Sign_Pos)						;If it was a low sign, see if person was above it
	add a,16-Sign_Height
	ld b,a
	ld a,(Person_Pos)
	add a,Person_Jump_Height
	cp b
	jr c,Passed_Low						;If they were, they missed it!
	jr Fall

Passed_High:							;Increase score if passed sign
	ld hl,(Score)
	ld bc,High_Sign_Score
	add hl,bc
	ld (Score),hl
	ret
Passed_Low:
	ld hl,(Score)
	ld bc,Low_Sign_Score
	add hl,bc
	ld (Score),hl
	ret

;///////////////////////////////////////////////Fall////////////////////////////////////////////////

Test_Fall:
	ld a,(Jump_State)					;Skip if person's in jump
	or a
	ret nz
	ld a,(Person_Pos+1)					;Look at relative positions of car/person
	add a,Person_Width
	ld b,a
	ld a,(Car_Pos+1)
	cp b
	ret c								;If person is to right of left edge of car, we're safe
	add a,8*(Car_Width+1)				;Get position of right edge of car
	cp 127
	jr c,Test_Fall_Skip
	sub 127
Test_Fall_Skip:
	ld c,a
	ld a,b
	sub Person_Width
	ld b,a
	ld a,c
	cp b								;If person is to left of right edge of car, we're safe again
	ret nc
Fall:
	ld a,1								;If not, indicate that person is falling
	ld (Fall_State),a
	ret

Handle_Fall:
	ld hl,Person_Fall_Sprite			;Show falling person
	ld (Person_Sprite),hl
	ld a,(Person_Pos)					;Move person down
	add a,Fall_Move
	ld (Person_Pos),a
	cp Person_Fall_Pos					;Is person at the bottom of fall?
	jp nc,End_Game						;If so, exit
Fall_End_Loop:
	ld a,(Do_Redraw)						;Otherwise, wait until the next redraw period
	or a
	jr z,Fall_End_Loop
	call Redraw
	jp Loop

End_Game:
	ld hl,Person_Blank_Sprite			;Erase person
	ld (Person_Sprite),hl
	ld hl,(Score)						;Is it a new high score?
	ld de,(High_Score)
	or a
	sbc hl,de
	jr nc,New_High_Score
	xor a
	jr End_High_Score
New_High_Score:
	ld hl,(Score)						;If so, save the current score as the new high
	ld (High_Score),hl
	di
	call save_data
	call _RAM_PAGE_1
	ei
	ld a,1
End_High_Score:
	ld (Patch7),a
End_Loop:								;Show "lose" screen
	ld a,(Do_Redraw)
	or a
	jr z,Continue_End_Loop
	call Redraw
	ld hl,Lose_Pos						;and text
	ld (_curRow),hl
	ld hl,Lose_Str
	call _puts
Patch7	equ	$+1
	ld a,0								;Is there a new high score?
	or a
	jr z,Continue_End_Loop
	ld hl,New_High_Pos					;If there is, say so!
	ld (_penCol),hl
	ld hl,New_High_Str
	call _vputs
Continue_End_Loop:
	ld a,%00111111						;Check for keypress
	call Get_Key
	cp $ff
	jp nz,Exit_Prog						;If a key was pressed, exit
	jr End_Loop							;Otherwise, loop again

;/////////////////////////////////////////////Routines//////////////////////////////////////////////

Get_Key:								;Get keypress from port
	out (1),a
	nop
	nop
	in a,(1)
	ret

Wait_For_No_Key:						;Wait until none of the main keys is being pressed
	ld a,%00111111
	call Get_Key
	cp $ff
	jr nz,Wait_For_No_Key
	ret

Shift:									;Rotate 2D gfx buffer at (hl) with height of b right one byte
	push bc
	push hl
	ld de,15
	add hl,de
	ld a,(hl)
	ld (Patch1),a
	push hl
	pop de
	dec hl
	ld bc,15
	lddr
	pop hl
Patch1	equ	$+1
	ld (hl),0
	ld bc,16
	add hl,bc
	pop bc
	djnz Shift
	ret

;This code by Matt Johnson
Open_Gray:
 ld hl,Vector_Table_Addr	         ; Source = $8E00 (Start of vector table)
 ld de,Vector_Table_Addr+1		     ; Destination = $8E01
 ld (hl),ISR_Addr/256                ; Set first byte so it is copied to all of them ($8F8F)
 ld bc,256                           ; 256 bytes (size of vector table)
 ldir                                ; (DE) <- (HL), BC=BC-1, Loop till B=0
 ld hl,int_copy                      ; Source = grayscale interrupt routine)
 ld de,ISR_Addr                      ; Destination = $8F8F (as specified in vector table)
 ld bc,int_end-int_start             ; Length of custom interrupt routine
 ldir                                ; (DE) <- (HL), BC=BC-1, Loop till B=0
 ld a,Vector_Table_Addr/256          ; Set up address of vector table
 ld i,a                              ;  MSB = $8E
                                     ;  LSB = (byte supplied by data bus - random)
 im 2                                ; Interrupt Mode 2
 ei                                  ; Enable Interrupts
 ret                                 ; Return to calling program

;From grayscale routines by Dux Gregis
Close_Gray:
    im 1
    ld a,$3C                ;make sure video mem is active
    out (0),a
clrMem:             ;clear video mem and graph mem
    call _clrLCD
clrGrf:             ;clear graph mem
    ld hl,_plotSScreen+6
    ld de,_plotSScreen+7
    ld (hl), 0
    ld bc,1024
    ldir
    ret

;Writeback routine
;written by Jonah Cohen <ComAsYuAre@aol.com>
;http://jonah.ticalc.org
;
;A highly optimized writeback routine for saving high scores and saved games.
;It saves all data between data_start and data_end.

save_data:
	ld hl,_asapvar			;hl->name of program
	rst 20h				;copy to OP1
	rst 10h				;_findsym
	xor a
	ld hl,data_start-_asm_exec_ram+4	;offset
	add hl,de			;hl=pointer to data in original prog
	adc a,b				;in case we overlapped pages
	call _SET_ABS_DEST_ADDR
	xor a				;no absolute addressing now
	ld hl,data_start		;get data from here
	call _SET_ABS_SRC_ADDR
	ld hl,data_end-data_start	;number of bytes to save
	jp _MM_LDIR_SET_SIZE		;copy data and return

;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
DisplayHL:
 ld c,'0'  				; save ascii value for zero
 ld de,_OP1+5			; point to end of the buffer
 xor a				; zero terminate string
 ld (de),a				; set last byte to zero
DisplayHLl:
 call _divHLby10		; next digit
 dec de				; next position
 add a,c				; convert to ascii
 ld (de),a				; save digit
 ld a,h				; load upper byte
 or l					; check with lower byte for zero
 jr nz,DisplayHLl			; loop
 ex de,hl				; point to buffer
 jp _vputs			; print number, we're done

;=========================================================================
; Sprite routines by David Phillips <david@acz.org>    http://www.acz.org/
;=========================================================================
; All sprites are 8x8 masked sprites
;
; 	PutSprite      -- black/white			      (sprite, mask)
; 	PutSpriteClip  -- black/white with clipping	  (sprite, mask)
; 	GraySprite     -- grayscale				      (dark, light, mask)
; 	GraySpriteClip -- grayscale with clipping	  (dark, light, mask)
;
; in:   hl = pointer to sprite, bc = x,y location
; out:  af, bc, de, hl, ix = trash
; size: 250 bytes
;=========================================================================
GrayDraw1 = $fc00					; [1024] dark grayscale plane
GrayDraw2 = $ca00					; [1024] light grayscale plane

; in: hl = 8x8 masked grayscale sprite, bc = x,y location
GraySprite:
 push hl							; push sprite pointer
 pop ix								; pop in ix
 call FindAddress					; calc address and offset
GraySpriteE:
 push hl							; save address offset
 push ix							; save sprite pointer
 ld de,GrayDraw1					; draw to first gray buffer
 add hl,de							; add offset to buffer start
 ld a,16							; grayscale sprites are 16 bytes
 ld (RenderSpriteM),a				; adjust sprite load instruction
 call RenderSprite					; draw the sprite
 ld a,8								; second sprite is 8 bytes from mask
 ld (RenderSpriteM),a				; adjust sprite load instruction back
 pop ix								; restore sprite pointer
 pop hl								; restore address offset
 ld de,GrayDraw2					; now draw to second gray buffer
 add hl,de							; add offset to second buffer start
 ld de,$0008						; sprites are 8 bytes
 add ix,de							; advance sprite pointer

; in: hl = address in buffer, c = offset, ix = sprite
RenderSprite:
RenderSpriteCV =$+1
 ld b,8								; set row counter to 8 bytes
RenderSpriteL:
 push bc							; save row counter and sprite offset
 ld a,(ix+8)						; load mask for this row
RenderSpriteM =$-1
 cpl								; invert the mask
 ld b,a								; save inverted mask
 ld d,(ix)							; load sprite byte for this row
 inc ix								; advance sprite pointer
 ld a,c								; set shift counter to sprite offset
 ld c,$ff							; set all bits of right mask byte
 ld e,0								; clear right byte of sprite
 or a								; is the sprite aligned?
 jr z,RenderSpriteD					; then skip shifting it
RenderSpriteS:
 srl d								; shift left sprite byte
 rr e								; into the right sprite byte
 scf								; set carry
 rr b 								; shift left mask byte
 rr c								; into the right mask byte
 dec a								; decrease shift counter
 jr nz,RenderSpriteS				; loop until counter is 0
RenderSpriteD:
RenderSpriteCL =$
 ld a,b								; load the left sprite byte
 and (hl)							; and it with the screen
 or d								; or it with the mask
 ld (hl),a							; set screen memory to left byte
 inc hl								; increase screen pointer
RenderSpriteCR =$
 ld a,c								; load the right sprite byte
 and (hl)							; and it with the screen
 or e								; or it with the mask
 ld (hl),a							; set screen memory to left byte
 ld de,15							; next row is 15 bytes ahead
 add hl,de							; move screen pointer to next row
 pop bc								; restore loop counter and shift mask
 djnz RenderSpriteL					; loop for all bytes in sprite
 ret								; done drawing the sprite

; in: bc = x,y location
; out: hl = offset of byte, c = offset in video byte
FindAddress:
 ld a,c								; get the y coord
 add a,a							; multiply by 2
 add a,a							; again by 4
 ld l,a								; save as low byte
 ld h,0								; clear high byte
 add hl,hl							; multiply by 8
 add hl,hl							; and again by 16
 ld a,b								; now get the x coord
 and %11111000						; clear lower 3 bits
 rra								; divide by 2
 rra								; this time by 4
 rra								; and finally by 8
 or l								; combine with low byte
 ld l,a								; save it
 ld a,b								; get x coord again
 and %00000111						; clear all but last 3 bits
 ld c,a								; save sprite offset in c
 ret								; return to caller

;------------------------------------------------------
;   FINDPIXEL by CLEM
;
;   117 cycles  34 bytes    Destroyes : bc
;------------------------------------------------------

FindPixel:
    ld h,63
    ld a,c
    add a,a
    add a,a
    ld l,a
    ld a,b
    rra
    add hl,hl
    rra
    add hl,hl
    rra
    or l
    ld l,a
    ld a,b
    and 7
    ld bc,FP_Bits
    add a,c
    ld c,a
    adc a,b
    sub c
    ld b,a
    ld a,(bc)

;Modification {
	push hl
	ld hl,8
	add hl,bc
	ld b,(hl)
	pop hl
	ld c,a
	ld a,h
Patch3	equ	$+1
	add a,0
	ld h,a
	ld a,c
    ret

FP_Bits:    .db $ff,$7f,$3f,$1f,$0f,$07,$03,$01
;}
.db $80,$40,$20,$10,$08,$04,$02,$01

#include "background.asm"
#include "person2.asm"
#include "person2run.asm"
#include "person2jump.asm"
#include "person2fall.asm"
#include "person2duck.asm"
#include "lowsign.asm"
#include "highsign.asm"
#include "blank.asm"

;Taken from grayscale in Diamonds by Jonah Cohen, which was taken from Bomberbloke :-)
int_copy:
.org ISR_Addr
int_start:
	exx
	ex af,af'
	in a,(3)
	bit 3,a
	jr z,no_down_left_bug
	and %11111110
	out (3),a
no_down_left_bug:
	bit 1,a
	jr z,no_swap
	ld hl,grayscale_counter
	inc (hl)
	ld a,(hl)
	cp 3
	ld a,$3c	;$3c
	jr c,set_video_port
	ld a,$0a	;$0a
	ld (hl),0
set_video_port:
	out (0),a
no_swap:
;	jp $3a
end_int:
	ld hl,shift
	dec (hl)
	jr nz,Skip_Shift
	ld (hl),Game_Speed
	ld a,1
	ld (Do_Redraw),a
Skip_Shift:
;Following code by Matt Johnson
    in a,(3)                            ; Get byte from Port 3
    rra                                 ; Check Bit 0 (ON interrupt status) by using carry
    ld  a,9
    adc a,0                             ; A = $09 if ON interrupt did not occur, $0A if (carry) so
    out (3),a                           ; Write either $09 or $0A to Port 3
    ld a,$0B                                                        
    out (3),a                           ; Write $0B to Port 3
    ex af,af'
    exx
    ei
    reti

grayscale_counter: .db 0
shift: .db Game_Speed

int_end:

.end